@@ -67,6 +67,10 @@
 #define HAVE_SMOOTH_SCROLLING 1
 #endif
 
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 18
+#define LogMessageVerbSigSafe xf86MsgVerb
+#endif
+
 #define EVDEV_MAXBUTTONS 32
 #define EVDEV_MAXQUEUE 32
 
@@ -153,11 +157,14 @@ typedef struct {
     int grabDevice;         /* grab the event device? */
 
     int num_vals;           /* number of valuators */
-    int axis_map[max(ABS_CNT, REL_CNT)]; /* Map evdev <axis> to index */
+    int num_mt_vals;        /* number of multitouch valuators */
+    int abs_axis_map[ABS_CNT]; /* Map evdev ABS_* to index */
+    int rel_axis_map[REL_CNT]; /* Map evdev REL_* to index */
     ValuatorMask *vals;     /* new values coming in */
     ValuatorMask *old_vals; /* old values for calculating relative motion */
     ValuatorMask *prox;     /* last values set while not in proximity */
     ValuatorMask *mt_mask;
+    ValuatorMask **last_mt_vals;
     int cur_slot;
     enum SlotState slot_state;
 #ifdef MULTITOUCH
@@ -175,9 +182,6 @@ typedef struct {
     int delta[REL_CNT];
     unsigned int abs_queued, rel_queued, prox_queued;
 
-    /* XKB stuff has to be per-device rather than per-driver */
-    XkbRMLVOSet rmlvo;
-
     /* Middle mouse button emulation */
     struct {
         BOOL                enabled;
@@ -231,12 +235,10 @@ typedef struct {
     OsTimerPtr reopen_timer;
 
     /* Cached info from device. */
-    char name[1024];
     unsigned long bitmask[NLONGS(EV_CNT)];
     unsigned long key_bitmask[NLONGS(KEY_CNT)];
     unsigned long rel_bitmask[NLONGS(REL_CNT)];
     unsigned long abs_bitmask[NLONGS(ABS_CNT)];
-    unsigned long led_bitmask[NLONGS(LED_CNT)];
     struct input_absinfo absinfo[ABS_CNT];
 
     /* minor/major number */
@@ -247,6 +249,8 @@ typedef struct {
     EventQueueRec           queue[EVDEV_MAXQUEUE];
 
     enum fkeymode           fkeymode;
+
+    char *type_name;
 } EvdevRec, *EvdevPtr;
 
 /* Event posting functions */
@@ -278,7 +282,6 @@ void EvdevMBEmuFinalize(InputInfoPtr);
 CARD32 Evdev3BEmuTimer(OsTimerPtr timer, CARD32 time, pointer arg);
 BOOL Evdev3BEmuFilterEvent(InputInfoPtr, int, BOOL);
 void Evdev3BEmuPreInit(InputInfoPtr pInfo);
-void Evdev3BEmuPreInit(InputInfoPtr);
 void Evdev3BEmuOn(InputInfoPtr);
 void Evdev3BEmuFinalize(InputInfoPtr);
 void Evdev3BEmuProcessRelMotion(InputInfoPtr pInfo, int dx, int dy);
